@@ -22,7 +22,7 @@ from werkzeug.serving import run_with_reloader
 from .namespace import Namespace
 from .test_client import SocketIOTestClient
 
-__version__ = '2.8.1'
+__version__ = '2.8.6'
 
 
 class _SocketIOMiddleware(socketio.Middleware):
@@ -36,6 +36,7 @@ class _SocketIOMiddleware(socketio.Middleware):
                                                   socketio_path)
 
     def __call__(self, environ, start_response):
+        environ = environ.copy()
         environ['flask.app'] = self.flask_app
         return super(_SocketIOMiddleware, self).__call__(environ,
                                                          start_response)
@@ -152,6 +153,8 @@ class SocketIO(object):
             if url:
                 if url.startswith('redis://'):
                     queue_class = socketio.RedisManager
+                elif url.startswith('zmq'):
+                    queue_class = socketio.ZmqManager
                 else:
                     queue_class = socketio.KombuManager
                 queue = queue_class(url, channel=channel,
@@ -224,7 +227,7 @@ class SocketIO(object):
                 self.server.on(message, _handler, namespace=namespace)
             else:
                 self.handlers.append((message, _handler, namespace))
-            return _handler
+            return handler
         return decorator
 
     def on_error(self, namespace=None):
@@ -687,7 +690,7 @@ def send(message, **kwargs):
                          include_self=include_self, callback=callback)
 
 
-def join_room(room):
+def join_room(room, sid=None, namespace=None):
     """Join a room.
 
     This function puts the user in a room, under the current namespace. The
@@ -702,18 +705,22 @@ def join_room(room):
             send(username + ' has entered the room.', room=room)
 
     :param room: The name of the room to join.
+    :param sid: The session id of the client. If not provided, the client is
+                obtained from the request context.
+    :param namespace: The namespace for the room. If not provided, the
+                      namespace is obtained from the request context.
     """
     socketio = flask.current_app.extensions['socketio']
-    socketio.server.enter_room(flask.request.sid, room,
-                               namespace=flask.request.namespace)
+    sid = sid or flask.request.sid
+    namespace = namespace or flask.request.namespace
+    socketio.server.enter_room(sid, room, namespace=namespace)
 
 
-def leave_room(room):
+def leave_room(room, sid=None, namespace=None):
     """Leave a room.
 
     This function removes the user from a room, under the current namespace.
-    The user and the namespace are obtained from the event context. This is
-    a function that can only be called from a SocketIO event handler. Example::
+    The user and the namespace are obtained from the event context. Example::
 
         @socketio.on('leave')
         def on_leave(data):
@@ -723,35 +730,47 @@ def leave_room(room):
             send(username + ' has left the room.', room=room)
 
     :param room: The name of the room to leave.
+    :param sid: The session id of the client. If not provided, the client is
+                obtained from the request context.
+    :param namespace: The namespace for the room. If not provided, the
+                      namespace is obtained from the request context.
     """
     socketio = flask.current_app.extensions['socketio']
-    socketio.server.leave_room(flask.request.sid, room,
-                               namespace=flask.request.namespace)
+    sid = sid or flask.request.sid
+    namespace = namespace or flask.request.namespace
+    socketio.server.leave_room(sid, room, namespace=namespace)
 
 
-def close_room(room):
+def close_room(room, namespace=None):
     """Close a room.
 
     This function removes any users that are in the given room and then deletes
-    the room from the server. This is a function that can only be called from
-    a SocketIO event handler.
+    the room from the server.
 
     :param room: The name of the room to close.
+    :param namespace: The namespace for the room. If not provided, the
+                      namespace is obtained from the request context.
     """
     socketio = flask.current_app.extensions['socketio']
-    socketio.server.close_room(room, namespace=flask.request.namespace)
+    namespace = namespace or flask.request.namespace
+    socketio.server.close_room(room, namespace=namespace)
 
 
-def rooms():
+def rooms(sid=None, namespace=None):
     """Return a list of the rooms the client is in.
 
     This function returns all the rooms the client has entered, including its
-    own room, assigned by the Socket.IO server. This is a function that can
-    only be called from a SocketIO event handler.
+    own room, assigned by the Socket.IO server.
+
+    :param sid: The session id of the client. If not provided, the client is
+                obtained from the request context.
+    :param namespace: The namespace for the room. If not provided, the
+                      namespace is obtained from the request context.
     """
     socketio = flask.current_app.extensions['socketio']
-    return socketio.server.rooms(flask.request.sid,
-                                 namespace=flask.request.namespace)
+    sid = sid or flask.request.sid
+    namespace = namespace or flask.request.namespace
+    return socketio.server.rooms(sid, namespace=namespace)
 
 
 def disconnect(silent=False):
